#include "aac/decoder.hh"

#include <string>
#include <fstream>
#include <iostream>

int usage(const char* program) {
    std::cerr << "usage: "
              << program << " "
              << "<input-file> "
              << "<output-file>"
              << std::endl;
    return 1;
}

int main(int argc, char** argv) {
    if (argc != 3) return usage(argv[0]);
    std::string input_path  { argv[1] },
                output_path { argv[2] };

    // Only allow opening of existing input
    // files, but allow non-existing output.
    std::ifstream input_file { input_path,   std::ios::binary};
    if (!input_file)  return usage(argv[0]);
    std::ofstream output_file { output_path, std::ios::binary};
    if (!output_file) return usage(argv[0]);

    aac::Statistics default_statistics; // Uniform.
    aac::Decoder ac_decoder { default_statistics };

    int decoder_status;
    aac::Byte byte { 0 };
    std::size_t head { 0 };
    aac::Byte bit_buffer[aac::MAX_PENDING];
    input_file.read((char*)&byte, sizeof(aac::Byte));

    do {
        std::size_t head_byte { (head >> 3) };
        while (ac_decoder.feed(byte, head - 8*head_byte))
            input_file.read((char*)&byte, sizeof(aac::Byte));
        decoder_status = ac_decoder.step(); // Feed the data!
        output_file.write((char*)&decoder_status, 1);
    } while (decoder_status != -1);

    return 0;
}
